% Michael Vodhanel

% Purpose: Used ICA to find independent components of DNA samples known
%				to contan cancer and compares the components with an unused
%				set of DNA samples known to have cancer and a set that is known
%				to be cancer free to see if the independent components are
%				indicators of bladder cancer.

% Input:
%	BladderData: The Bladder DNA data set 

% Output:
%	Phi: The characteristic images (modes) of the DNA data set with cancer to the screen
%	U: The independent components of the modes found by ICA to the screen
%  DrawCovPlot: Covariance plots of various sets
%	Tum_Mean: Means of the covariance of the set known to have cancer wih the indep comps
%	Tum_StdDev = Standard deviations of the covariance of the set known to have cancer wih the indep comps
%	Tum_Skew = Skews of the covariance of the set known to have cancer wih the indep comps
%	Norm_Mean = Means of the covariance of the set known to not have cancer wih the indep comps
%	Norm_StdDev = Standard deviations of the covariance of the set known to not have cancer wih the indep comps
%	Norm_Skew = Skews of the covariance of the set known to not have cancer wih the indep comps

clear all;
close all;


% Set Parameters
Bladder = 0;
Center = 0;
if (Bladder == 1)
   NumOfModes = 4; % 4 seems to give best results, 5 and 6 work wall also
   DataFile = 'Bladder_Data.txt';
   NormalIndex = [103:124];
   TumorIndexAll = [1:6,8:102,125,126];
   Perm = randperm(103);
   UsedIndex = Perm(1:80);
   UnusedIndex = Perm(81:103);
   %UsedIndex = [1:80];
   %UnusedIndex = [81:103];
   % Make up a draw height and width for viewing results
	Height = 6688 / 32;
	Width = 6688 / 209;
else
   NumOfModes = 4;
   DataFile = 'Liver_Data.txt';
   %adenomaIndex=[1,2,3];
	%cellLineIndex=[113:115, 199:205]; FNHIndex=[5,6,7,8]; Metastatic=[192:198, 206, 207];
	NormalIndex=[116:191];
   TumorIndexAll=[4,9:112];
   Perm = randperm(105);
   UsedIndex = Perm(1:80);
   UnusedIndex = Perm(81:105);
   % Make up a draw height and width for viewing results
	Height = 5520 / 69;
	Width = 5520 / 80;
end

% Load data from file
fprintf('Loading cancer data\n\n')
BladderData = load(DataFile);
DNA_Size = size(BladderData, 1);

% seperate data into normal and tumor sets
NumOfNormal = length(NormalIndex)
NumOfTumor = length(TumorIndexAll)

Normal = BladderData(:, NormalIndex);
Tumor = BladderData(:, TumorIndexAll);
clear BladderData;


% Get four characteristic images for the tumor data (the first four modes) 
% using the conventional method
fprintf('Finding characteristics images for tumors \n\n')
Phi = zeros(DNA_Size, NumOfModes);
Phi = POD_Modes(Tumor(:,UsedIndex), NumOfModes, Center);

% Draw modes covariance with used samples
figure(10); clf
DrawCovPlot(Phi, Tumor(:,UsedIndex), 10, 'PCA: Covariance of Tumor Used Samples With Phi', 'k')
% Draw Independent Component's covariance with unused samples with tumors
%figure(11); clf
DrawCovPlot(Phi, Tumor(:,UnusedIndex), 11, 'PCA: Covariance of Unused Samples With Phi', 'b')
% Draw Independent Component's covariance with cancer free samples
DrawCovPlot(Phi, Normal, 12, 'PCA: Covariance of Normal Samples With Phi', 'r')

% Draw characteristic images which are also the signals into ICA
figure(1);clf
for i=1:NumOfModes
    subplot(NumOfModes, 1, i);
    Title = ['Signal ' num2str(i)];
    Draw_Sub_Fig(Phi(:,i), Height, Width, Title, 'f');
end

% Use ICA to find the independent components
% Transpose the data since ICA uses rows instead of columns for input
fprintf('Performing ICA \n\n')
U = ICA(Phi');

% Draw Independent Component's covariance with used samples
figure(13); clf
DrawCovPlot(U', Tumor(:,UsedIndex), 13, 'ICA: Covariance of Tumor Used Samples With U', 'k')
% Draw Independent Component's covariance with unused samples with tumors
%figure(14); clf
DrawCovPlot(U', Tumor(:,UnusedIndex), 14, 'ICA: Covariance of Tumor Unused Samples With U', 'b')
% Draw Independent Component's covariance with cancer free samples
DrawCovPlot(U', Normal, 15, 'ICA: Covariance of Normal Samples With U', 'r')

% Draw independent components
figure(2);clf
for i=1:NumOfModes
    subplot(NumOfModes, 1, i);
    Title = ['Inpendent Component ' num2str(i)];
    Draw_Sub_Fig(U(i,:), Height, Width, Title, 'f');
end
 
% Get statistics for comparisons of the independent components with
% 1) The data set of unused DNA samples known to have cancer
% 2) The data set of DNA samples known to not have cancer
CompareCanc = zeros(NumOfModes, length(UnusedIndex));
CompareNorm = zeros(NumOfModes, NumOfNormal);
for i = 1:NumOfModes
   for j = 1:length(UnusedIndex)
      CompareCanc(i,j) = dot(U(i,:), Tumor(:,UnusedIndex(j))') / (norm(U(i,:)) * norm(Tumor(:,UnusedIndex(j))));
   end
   for j = 1:NumOfNormal 
       CompareNorm(i,j) = dot(U(i,:), Normal(:,j)') / (norm(U(i,:)) * norm(Normal(:,j)));
   end
end
Tum_Mean = mean(CompareCanc')
Tum_StdDev = std(CompareCanc')
Tum_Skew = sum((CompareCanc' - repmat(Tum_Mean, length(UnusedIndex), 1)).^3) ./ Tum_StdDev.^3
Norm_Mean = mean(CompareNorm')
Norm_StdDev = std(CompareNorm')
Norm_Skew = sum((CompareNorm' - repmat(Norm_Mean, NumOfNormal, 1)).^3) ./ Norm_StdDev.^3