% Michael Vodhanel

% Purpose: Used ICA to find independent components of DNA samples known
%				to contan cancer and compares the components with an unused
%				set of DNA samples known to have cancer and a set that is known
%				to be cancer free to see if the independent components are
%				indicators of bladder cancer.

% Input:
%	BladderData: The Bladder DNA data set 

% Output:
%	Phi: The characteristic images (modes) of the DNA data set with cancer to the screen
%	U: The independent components of the modes found by ICA to the screen
%  DrawCovPlot: Covariance plots of various sets
%	Stat Analysis: Graphical analsysis of results to the screen

function Bladder_ICA_Modes_Itt(NumOfItt, NumOfModes, Center)

% % NumOfItt=5; NumOfModes=4; Center=0;
% Set Parameters
Bladder = 0;
%Center = 0;
%NumOfItt = 5;
if (Bladder == 1)
    %   NumOfModes = 4; % 4 seems to give best results, 5 and 6 work wall also
    DataFile = 'Bladder_Data.txt';
    NormalIndex = [103:124];
    TumorIndexAll = [1:6,8:102,125,126];
    %UsedIndex = [1:80];
    %UnusedIndex = [81:103];
    % Make up a draw height and width for viewing results
    Height = 6688 / 32;
    Width = 6688 / 209;
elseif (Bladder == 0)
    %   NumOfModes = 5;
    DataFile = 'Liver_Data.txt';
    %adenomaIndex=[1,2,3];
    %cellLineIndex=[113:115, 199:205]; FNHIndex=[5,6,7,8]; Metastatic=[192:198, 206, 207];
    NormalIndex=[116:191];
    TumorIndexAll=[4,9:112];
    % Make up a draw height and width for viewing results
    Height = 5520 / 69;
    Width = 5520 / 80;
else
    %   NumOfModes = 4; % 4 seems to give best results, 5 and 6 work wall also
    DataFile = 'Bladder_Data.txt';
    NormalIndex = [53:124];
    TumorIndexAll = [1:6,8:52,125,126];
    %UsedIndex = [1:80];
    %UnusedIndex = [81:103];
    % Make up a draw height and width for viewing results
    Height = 6688 / 32;
    Width = 6688 / 209;
end

% Load data from file
fprintf('Loading cancer data\n\n')
BladderData = load(DataFile);
DNA_Size = size(BladderData, 1);

% seperate data into normal and tumor sets
NumOfNormal = length(NormalIndex)
NumOfTumor = length(TumorIndexAll)
NumOfSamples = NumOfNormal + NumOfTumor;

Normal = BladderData(:, NormalIndex);
Tumor = BladderData(:, TumorIndexAll);
clear BladderData;

POD_Cor_Mat = zeros(NumOfNormal + NumOfTumor, NumOfItt, NumOfModes);
ICA_Cor_Mat = zeros(NumOfNormal + NumOfTumor, NumOfItt, NumOfModes);
Fitness=0;FitnessValue=[];BestIteration=1;
for k = 1:NumOfItt
    if (Bladder == 1)
        Perm = randperm(103);
        UsedIndex = Perm(1:85);
        UnusedIndex = Perm(86:103);
    elseif (Bladder == 0)
        Perm = randperm(105);
        UsedIndex = Perm(1:85);
        UnusedIndex = Perm(86:105);
    else
        Perm = randperm(53);
        UsedIndex = Perm(1:25);
        UnusedIndex = Perm(25:53);
    end
    NumUsed = length(UsedIndex);
    NumUnused = length(UnusedIndex);
    
    % Get four characteristic images for the tumor data (the first four modes) 
    % using the conventional method
    fprintf('Finding characteristics images for tumors \n\n')
    Phi = zeros(DNA_Size, NumOfModes);
    Phi = POD_Modes(Tumor(:,UsedIndex), NumOfModes, Center);
    for i=1:NumOfModes
        Phi(:,i)=Phi(:,i)/norm(Phi(:,i));
    end
    
    % Use ICA to find the independent components
    % Transpose the data since ICA uses rows instead of columns for input
    fprintf('Performing ICA \n\n')
    U = ICA(Phi')';
    for i=1:NumOfModes
        U(:,i)=U(:,i)/norm(U(:,i));
    end
    
    if k == 1
        U_First = U;
    else
        U = OrderComponents(U, U_First);
    end
    
    % Find correlations for DNA Samples with POD modes
    for j = 1:NumOfTumor
        for i = 1:NumOfModes
            POD_Cor_Mat(j, k, i) = dot(Tumor(:,j), Phi(:,i)) / norm(Phi(:,i));
        end
    end
    for j = 1:NumOfNormal
        for i = 1:NumOfModes
            POD_Cor_Mat(j+NumOfTumor, k, i) = dot(Normal(:,j), Phi(:,i)) / norm(Phi(:,i));
        end
    end
    
    % Find correlations for DNA Samples with ICA signals
    for j = 1:NumOfTumor
        for i = 1:NumOfModes
            ICA_Cor_Mat(j, k, i) = dot(Tumor(:,j), U(:,i)) / norm(U(:,i));
        end
    end
    for j = 1:NumOfNormal
        for i = 1:NumOfModes
            ICA_Cor_Mat(j+NumOfTumor, k, i) = dot(Normal(:,j), U(:,i)) / norm(U(:,i));
        end
    end

    % Calculate min and max values in the covariance matrices to make plots with the same
    % scale possible
    MaxCor=max(max(max(max(POD_Cor_Mat))), max(max(max(ICA_Cor_Mat))));
    MinCor=min(min(min(min(POD_Cor_Mat))), min(min(min(ICA_Cor_Mat))));
    MaxCorPOD=max(max(max(POD_Cor_Mat)));
    MinCorPOD=min(min(min(POD_Cor_Mat)));
    MaxCorICA=max(max(max(ICA_Cor_Mat)));
    MinCorICA=min(min(min(ICA_Cor_Mat)));
    %Normalize for plotting purpose comparing with POD/ICA
    TempTumor=Tumor;
    for i=1:NumOfTumor
        TempTumor(:,i)=Tumor(:,i)/norm(Tumor(:,i));
    end
    MaxTumor=max(max(max(TempTumor)));
    MinTumor=min(min(min(TempTumor)));
    MaxPOD=max(max(max(Phi)));
    MinPOD=min(min(min(Phi)));
    MaxICA=max(max(max(U)));
    MinICA=min(min(min(U)));
    MaxData=max([MaxPOD,MaxICA,MaxTumor]);
    MinData=min([MinPOD,MinICA,MinTumor]);
    [XX,YY]=meshgrid([10:11],[20000 20000]);

    %plotting if doing better in terms of ICA projection
    tempFitness=sum(ICA_Cor_Mat(1:NumOfTumor, k, 1))-sum(ICA_Cor_Mat(NumOfTumor+1:NumOfSamples, k, 1));
    FitnessValue=[FitnessValue tempFitness];
    if Fitness < tempFitness
        BestIteration=k;
        Fitness = tempFitness;
        
        % plot PCA modes for first iteration
        figure(40); clf
        for i = 1:NumOfModes
            subplot(1, NumOfModes, i);
            surf(XX,YY,[[MaxData MaxData]; [MinData MinData]]);hold on
            surf(reshape(Phi(:, i),Height, Width));hold off 
            title(['\Phi^{POD}_' num2str(i)]);
            axis([1, Width,1,Height,MinData,MaxData]);view(2);
            set(gca,'Xticklabel',[],'Yticklabel',[])
            colorbar;
        end
        drawnow;print -djpeg POD.jpeg
        
        % plot ICA modes for first iteration
        figure(41); clf
        for i = 1:NumOfModes
            subplot(1, NumOfModes, i);
            surf(XX,YY,[[MaxData MaxData]; [MinData MinData]]);hold on
            surf(reshape(U(:, i),Height, Width));hold off 
            title(['\Phi^{ICA}_' num2str(i)]);
            axis([1, Width,1,Height,MinData,MaxData]);view(2);
            set(gca,'Xticklabel',[],'Yticklabel',[])
            colorbar;
        end
        drawnow;print -djpeg ICA.jpeg
        % plot sample snapshots
        figure(42); clf
        for i = 1:NumOfModes
            subplot(1, NumOfModes, i);
            surf(XX,YY,[[MaxData MaxData]; [MinData MinData]]);hold on
            surf(reshape(TempTumor(:,UsedIndex(i)),Height, Width));hold off 
            title(['Tumor_' num2str(i)]);
            axis([1, Width,1,Height,MinData,MaxData]);view(2);
            set(gca,'Xticklabel',[],'Yticklabel',[])
            colorbar;
        end
        drawnow;print -djpeg TUMOR.jpeg
        % Plot normal analysis
        PlotNormalAnalysis(sum(POD_Cor_Mat(:, k, :),2), ICA_Cor_Mat(:, k, 1), NumOfTumor, MinCor, MaxCor, 46);
%        PlotNormalAnalysis(POD_Cor_Mat(:, k, 1), ICA_Cor_Mat(:, k, 1), NumOfTumor, MinCor, MaxCor, 46);
        set(gcf,'Name',['Best Iteration = ' num2str(BestIteration)])
    end
    
    
end

% Draw correlation matrix for samples with POD and ICA
figure(43); clf
for i = 1:NumOfModes
    % Draw correlation matrix for samples with POD
    subplot(2, NumOfModes, i);
    surf(XX,YY,[[MaxCor MaxCor]; [MinCor MinCor]]);hold on
    surf(POD_Cor_Mat(:,:,i));hold off 
    title(['\Phi^{POD}_' num2str(i) ' Correlation']);
    axis tight;
    axis([1,NumOfItt,1,NumOfSamples,MinCor,MaxCor]);view(2);
    set(gca,'Xticklabel',[]);xlabel([num2str(NumOfItt) ' trials'])
    colorbar;
    
    % Draw correlation matrix for samples with ICA
    subplot(2, NumOfModes, NumOfModes + i);
    surf(XX,YY,[[MaxCor MaxCor]; [MinCor MinCor]]);hold on
    surf(ICA_Cor_Mat(:,:,i));hold off
    title(['\Phi^{ICA}_' num2str(i) ' Correlation']);
    axis tight;
    axis([1,NumOfItt,1,NumOfSamples,MinCor,MaxCor]);view(2);
    set(gca,'Xticklabel',[]);xlabel([num2str(NumOfItt) ' trials'])
    colorbar;
end
drawnow;print -djpeg POD_COV.jpeg

% Draw correlation matrix for samples with POD and ICA
figure(53); clf
plot(FitnessValue);xlabel([num2str(NumOfItt) ' trials']);ylabel('Total Covariance')
title(['Best Iteration = ' num2str(BestIteration)])
drawnow;print -djpeg CovVsIteration.jpeg

% Draw Linear Combinations of Covariances
POD_Cor_Mat_Combined = zeros(NumOfNormal + NumOfTumor, NumOfItt);
ICA_Cor_Mat_Combined = zeros(NumOfNormal + NumOfTumor, NumOfItt);
for i = 1:NumOfModes
    POD_Cor_Mat_Combined = POD_Cor_Mat_Combined + POD_Cor_Mat(:, :, i);
    ICA_Cor_Mat_Combined = ICA_Cor_Mat_Combined + ICA_Cor_Mat(:, :, i);
end
    MaxComCor=max(max(max(POD_Cor_Mat_Combined)),max(max(ICA_Cor_Mat_Combined)));
    MinComCor=min(min(min(POD_Cor_Mat_Combined)),min(min(ICA_Cor_Mat_Combined)));

% Draw linear combination of covariances with POD modes 
figure(45); clf
subplot(1, 2, 1);
surf(XX,YY,[[MaxComCor MaxComCor]; [MinComCor MinComCor]]);hold on
surf(POD_Cor_Mat_Combined);hold off
title('POD Modes Combined')
axis([1,NumOfItt,1,NumOfSamples,MinComCor,MaxComCor]);view(2);
colorbar; 
% Draw linear combination of covariances with ICA modes 
subplot(1, 2, 2);
surf(XX,YY,[[MaxComCor MaxComCor]; [MinComCor MinComCor]]);hold on
surf(ICA_Cor_Mat_Combined);hold off
title('ICA Modes Combined')
axis([1,NumOfItt,1,NumOfSamples,MinComCor,MaxComCor]);view(2);
colorbar; 

