% Michael Vodhanel

% Purpose: Used ICA to find independent components of DNA samples known
%				to contan cancer and compares the components with an unused
%				set of DNA samples known to have cancer and a set that is known
%				to be cancer free to see if the independent components are
%				indicators of bladder cancer.

% Input:
%	BladderData: The Bladder DNA data set

% Output:
%	Phi: The characteristic images (modes) of the DNA data set with cancer to the screen
%	U: The independent components of the modes found by ICA to the screen
%  DrawCovPlot: Covariance plots of various sets
%	Tum_Mean: Means of the covariance of the set known to have cancer wih the indep comps
%	Tum_StdDev = Standard deviations of the covariance of the set known to have cancer wih the indep comps
%	Tum_Skew = Skews of the covariance of the set known to have cancer wih the indep comps
%	Norm_Mean = Means of the covariance of the set known to not have cancer wih the indep comps
%	Norm_StdDev = Standard deviations of the covariance of the set known to not have cancer wih the indep comps
%	Norm_Skew = Skews of the covariance of the set known to not have cancer wih the indep comps

clear all;
close all;

% Set Parameters
Bladder = 0;
Center = 0;
NumOfItt = 10;
if (Bladder == 1)
    NumOfModes = 4; % 4 seems to give best results, 5 and 6 work wall also
    DataFile = 'Bladder_Data.txt';
    NormalIndex = [103:124];
    TumorIndexAll = [1:6,8:102,125,126];
    %UsedIndex = [1:80];
    %UnusedIndex = [81:103];
    % Make up a draw height and width for viewing results
    Height = 6688 / 32;
    Width = 6688 / 209;
elseif (Bladder == 0)
    NumOfModes = 1; %3;
    DataFile = 'Liver_Data.txt';
    %adenomaIndex=[1,2,3];
    %cellLineIndex=[113:115, 199:205]; FNHIndex=[5,6,7,8]; Metastatic=[192:198, 206, 207];
    NormalIndex=[116:191];
    TumorIndexAll=[4,9:112];
    % Make up a draw height and width for viewing results
    Height = 5520 / 69;
    Width = 5520 / 80;
else
    NumOfModes =1; % 4; % 4 seems to give best results, 5 and 6 work wall also
    DataFile = 'Bladder_Data.txt';
    NormalIndex = [53:124];
    TumorIndexAll = [1:6,8:52,125,126];
    %UsedIndex = [1:80];
    %UnusedIndex = [81:103];
    % Make up a draw height and width for viewing results
    Height = 6688 / 32;
    Width = 6688 / 209;
end

% Load data from file
fprintf('Loading cancer data\n\n')
BladderData = load(DataFile);
DNA_Size = size(BladderData, 1);

% seperate data into normal and tumor sets
NumOfNormal = length(NormalIndex)
NumOfTumor = length(TumorIndexAll)

Normal = BladderData(:, NormalIndex);
Tumor = BladderData(:, TumorIndexAll);
clear BladderData;

POD_Cor_Mat = zeros(NumOfNormal + NumOfTumor, NumOfItt);
ICA_Cor_Mat = zeros(NumOfNormal + NumOfTumor, NumOfItt);

for k = 1:NumOfItt
    if (Bladder == 1)
        Perm = randperm(103);
        UsedIndex = Perm(1:85);
        UnusedIndex = Perm(86:103);
    elseif (Bladder == 0)
        Perm = randperm(105);
        UsedIndex = Perm(1:85);
        UnusedIndex = Perm(86:105);
    else
        Perm = randperm(53);
        UsedIndex = Perm(1:25);
        UnusedIndex = Perm(25:53);
    end
    NumUsed = length(UsedIndex);
    NumUnused = length(UnusedIndex);

    % Get four characteristic images for the tumor data (the first four modes)
    % using the conventional method

    NumUsed=[1:85];
    fprintf('Finding characteristics images for tumors \n\n')
    Phi = zeros(DNA_Size, NumOfModes);
    %%%%Phi = POD_Modes(Tumor(:,UsedIndex), NumOfModes, Center);
    Phi = POD_Modes(Tumor(:,1:85), NumOfModes, Center);

    % Use ICA to find the independent components
    % Transpose the data since ICA uses rows instead of columns for input
    fprintf('Performing ICA \n\n')
    U = ICA(Phi')';

    % Find correlations for DNA Samples with POD modes
    for j = 1:NumOfTumor
        for i = 1:NumOfModes
            POD_Cor_Mat(j, k) = POD_Cor_Mat(j, k) + dot(Tumor(:,j), Phi(:,i)) / dot(Phi(:,i), Phi(:,i));
        end
    end
    if 1 == 0
        for j = 1:NumUnused
            for i = 1:NumOfModes
                POD_Cor_Mat(j+NumUsed, k) = POD_Cor_Mat(j+NumUsed, k) + dot(Tumor(:,UnusedIndex(j)), Phi(:,i)) / dot(Phi(:,i), Phi(:,i));
            end
        end
    end
    for j = 1:NumOfNormal
        for i = 1:NumOfModes
            POD_Cor_Mat(j+NumOfTumor, k) = POD_Cor_Mat(j+NumOfTumor, k) + dot(Normal(:,j), Phi(:,i)) / dot(Phi(:,i), Phi(:,i));
        end
    end

    % Find correlations for DNA Samples with ICA signals
    for j = 1:NumOfTumor
        for i = 1:NumOfModes
            ICA_Cor_Mat(j, k) = ICA_Cor_Mat(j, k) + dot(Tumor(:,j), U(:,i)) / dot(U(:,i), U(:,i));
        end
    end
    if 1 == 0
        for j = 1:NumUnused
            for i = 1:NumOfModes
                ICA_Cor_Mat(j+NumUsed, k) = ICA_Cor_Mat(j+NumUsed, k) + dot(Tumor(:,UnusedIndex(j)), U(:,i)) / dot(U(:,i), U(:,i));
            end
        end
    end
    for j = 1:NumOfNormal
        for i = 1:NumOfModes
            ICA_Cor_Mat(j+NumOfTumor, k) = ICA_Cor_Mat(j+NumOfTumor, k) + dot(Normal(:,j), U(:,i)) / dot(U(:,i), U(:,i));
        end
    end

end

% Draw correlation matrix for samples with POD
figure(25); clf
surf(POD_Cor_Mat);
view(2);
colorbar;

% Draw correlation matrix for samples with POD
figure(26); clf
surf(ICA_Cor_Mat);
view(2);
colorbar;

% Draw binary figures
%figure(27); clf
%surf(ICA_Cor_Mat>-30);    % BUG nabbasi
%view(2);
%colorbar;


%figure(28); clf
%surf(POD_Cor_Mat>-.38);
%view(2);
%colorbar;