% Purpose: Uses the POD method to find the Dominant Image

% Input:
%	Snapshots: The 4 dimensional vector containing the noisy snapshots
%  Noise: The Noise snapshots used by the generalized eigenvalue schemes
%	Scheme: The scheme to be used
%  Nite: The number of itterations to be performed

% Output:
%	Phi: The characteristic (dominant) image generated from the snapshots
%  weights: The weights found by the POD method to create the dominant image

function [Phi, weights] = POD(Snapshots, Noise, Scheme, Nite)

% conventional POD method
if strcmp(lower(Scheme), 'c')
    
    Ns = size(Snapshots, 2);
    Theta = zeros(Ns);
    Phi = zeros(size(Snapshots,1), 1);

    for i = 1:Ns
        for j = i:Ns
            Theta(i, j) = sum(Snapshots(:, i).* Snapshots(:, j) );
        end
    end
    for i = 2:Ns
        for j = 1:i-1
            Theta(i, j) = Theta(j, i);
        end
    end
    
    [V,D]=eig(Theta);
    %D(Ns,Ns)
    w = V(:,Ns);
    w = w / sum(abs(w)); %w = w / sum(w);
    
    weights = w;
    for i = 1:Nite
        weights = [weights w];    
    end
    
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end
    
% power POD method
elseif strcmp(lower(Scheme), 'p')
    
    Ns = size(Snapshots, 2);
    Theta = zeros(Ns);
    w = ones(Ns, 1);
    w = w / sum(abs(w));
    Phi = zeros(size(Snapshots,1), 1);
    weights = w;
    
    for i = 1:Ns
        for j = i:Ns
            Theta(i, j) = sum(Snapshots(:, i).* Snapshots(:, j) );
        end
    end
    for i = 2:Ns
        for j = 1:i-1
            Theta(i, j) = Theta(j, i);
        end
    end
    
    for i=1:Nite
        w = Theta * w;
        w = w / sum(abs(w));w = w / sum(w);
        weights = [weights w];
    end
    %w = PowerMethod(Theta, w, Nite);
    %w = w / sum(abs(w));w = w / sum(w);
    %weights = w;
    
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end
    
% lump POD method
elseif strcmp(lower(Scheme), 'l')
    
    Ns = size(Snapshots, 2);
    w = ones(Ns, 1);
    w = w / sum(abs(w));
    weights = w;

    for iter = 1:Nite
        %Lump = zeros(size(Snapshots, 1), 1);
        %for k = 1:Ns
        %    Lump = Lump + w(k) * Snapshots(:, k);
        %end
        Lump = Snapshots * w; 
        
        for k = 1:Ns
            w(k) = sum( Snapshots(:, k) .* Lump );
        end
        w = w / sum(abs(w)); w = w / sum(w);
        weights = [weights w];
    end
    
    Phi = zeros(length(Snapshots), 1);
    
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end

% conventional noise minimization method
elseif strcmp(lower(Scheme), 'cnm')
    
    Ns = size(Snapshots, 2);
    ThetaN = zeros(Ns);
    Phi = zeros(size(Snapshots, 1), 1);
    
    for i = 1:Ns
        for j = i:Ns
            ThetaN(i, j) = sum(Noise(:, i).* Noise(:, j) );
        end
    end
    for i = 2:Ns
        for j = 1:i-1
            ThetaN(i, j) = ThetaN(j, i);
        end
    end
    
    [V,D]=eig(ThetaN);
    w = V(:,1);
    w = w / sum(abs(w)); w = w / sum(w);
    
    weights = w;
    for i = 1:Nite
        weights = [weights w];    
    end
    
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end

% noise minimization power method
elseif strcmp(lower(Scheme), 'nmp')
    
    Ns = size(Snapshots, 2);
    ThetaN = zeros(Ns);
    w = ones(Ns, 1);
    w = w / sum(abs(w));
    Phi = zeros(size(Snapshots,1), 1);
    weights = w;
    
    for i = 1:Ns
        for j = i:Ns
            ThetaN(i, j) = sum(Noise(:, i).* Noise(:, j) );
        end
    end
    for i = 2:Ns
        for j = 1:i-1
            ThetaN(i, j) = ThetaN(j, i);
        end
    end
    
    A = inv(ThetaN);
    for i=1:Nite
        w = A * w;
        w = w / sum(abs(w));w = w / sum(w);
        weights = [weights w];
    end
    
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end
    
% noise minimization lump method
elseif strcmp(lower(Scheme), 'nm')
    
    Ns = size(Snapshots, 2);
    w = ones(Ns, 1) / Ns;
    weights = w;
    Phi = zeros(size(Snapshots, 1), 1);
    
    for k = 1:Nite
       w = GaussOne(Noise, w, w);
       w = w / sum(abs(w));
       w = w / sum(w);
       weights = [weights w];
    end
    
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end
    
% adaptive noise minimization method
elseif strcmp(lower(Scheme), 'anm')
    
    Ns = size(Snapshots, 2);
    height = size(Snapshots, 1);
    subheight = floor(height / Nite);
    w = ones(Ns, 1) / Ns;
    w_old = w;
    weights = w / sum(abs(w));
    Phi = zeros(size(Snapshots, 1), 1);
    
    for k = 1:Nite
       w_new = GaussOne(Noise((k-1)*subheight+1:k*subheight, :), w, k * w - (k-1) * w_old);
       w_old = w;
       w = w_new;
       %w_old = w;
       %w = w / sum(abs(w));
       %w = w / sum(w);
       weig = w/sum(abs(w)); weig = weig / sum(weig);
       weights = [weights weig];
    end
    
    w = w / sum(abs(w));
    w = w / sum(w);
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end     
    
% adaptive POD method
elseif strcmp(lower(Scheme), 'a')
    
    Ns = size(Snapshots, 2);
    height = size(Snapshots, 1);
    subheight = floor(height / Nite);
    w = ones(Ns, 1) / Ns;
    weights = w / sum(abs(w));
    Phi = zeros(size(Snapshots, 1), 1);
    
    % perform all the full sized itterations of the adaptive method
%for j = 1:Nite
    for i = 1:floor(height / subheight)
        % do one itteration of power method with the new theta
        % equivilant - matrix multiplication is same as adding weighted columns 
        %for j = 1:Nite
            c =  Snapshots((i-1)*subheight+1:i*subheight, :) * w;
 
            % add new sum to accumulated eigenvector with appropriate weights
            for k = 1:Ns
                s = 1 / subheight * sum(Snapshots((i-1)*subheight+1:i*subheight, k) .* c);
                w(k) = i/(i+1) * w(k) + 1/(i+1) * s;   
            end
            %w = w/sum(abs(w));
            %end
        % save the weight at this itteration
       weights = [weights w/sum(abs(w))];
    end
     
     %end
    % perform the last itteration in case subheight does not divide height
    if height - i*subheight ~= 0
        c = zeros(height - i*subheight, 1);

        c = Snapshots(i*subheight+1:i*subheight+length(c), :) * w;
        c = c * (height - i*subheight)/subheight;  
        % add new sum to accumulated eigenvector with appropriate weights
        % adjust weight to accomidate the smaller size of the last peice
        for k = 1:Ns
            s = 1 / subheight * sum(Snapshots(i*subheight+1:height, k) .* c);
				w(k) = (i+1)/(i+2)*w(k) + 1/(i+2)*s;   
        end
        weights = [weights w/sum(abs(w))];
    end
    
    % normalize weights
    w = w / sum(abs(w));
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end
    
% generalized conventional POD method
elseif strcmp(lower(Scheme), 'gc')
    
    Ns = size(Snapshots, 2);
    ThetaS = zeros(Ns);
    ThetaN = zeros(Ns);
    w = ones(Ns, 1) / Ns;
    Phi = zeros(size(Snapshots,1), 1);
    
    for i = 1:Ns
        for j = i:Ns
            ThetaS(i, j) = sum(Snapshots(:, i).* Snapshots(:, j) );
        end
    end
    for i = 2:Ns
        for j = 1:i-1
            ThetaS(i, j) = ThetaS(j, i);
        end
    end
    for i = 1:Ns
        for j = i:Ns
            ThetaN(i, j) = sum(Noise(:, i).* Noise(:, j) );
        end
    end
    for i = 2:Ns
        for j = 1:i-1
            ThetaN(i, j) = ThetaN(j, i);
        end
    end
    A=inv(ThetaN)*ThetaS;
    [V,D]=eig(A); %V(:,1)'*A*V(:,1)/norm(V(:,1))^2 % Dominant eigvalue????
    %D(1,1)
    w = -sign(D(1,1))*V(:,1);
    w = w / sum(abs(w));w = w / sum(w);
    
    weights = w;
    for i = 1:Nite
        weights = [weights w];
    end
    
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end
    
    
% generalized power POD method
elseif strcmp(lower(Scheme), 'gp')
    
    Ns = size(Snapshots, 2);
    ThetaS = zeros(Ns);
    ThetaN = zeros(Ns);
    w = ones(Ns, 1) / Ns;
    Phi = zeros(size(Snapshots,1), 1);
    weights = w;
    for i = 1:Ns
        for j = i:Ns
            ThetaS(i, j) = sum(Snapshots(:, i).* Snapshots(:, j) );
        end
    end
    for i = 2:Ns
        for j = 1:i-1
            ThetaS(i, j) = ThetaS(j, i);
        end
    end
    for i = 1:Ns
        for j = i:Ns
            ThetaN(i, j) = sum(Noise(:, i).* Noise(:, j) );
        end
    end
    for i = 2:Ns
        for j = 1:i-1
            ThetaN(i, j) = ThetaN(j, i);
        end
    end
    A=inv(ThetaN) * ThetaS;
    for k=1:Nite
        w = A*w;
        w = w / sum(abs(w));w = w / sum(w);
        weights = [weights w];
    end
    %     w = PowerMethod(inv(ThetaN) * ThetaS, w, Nite);
    %     w = w / sum(abs(w));w = w / sum(w);
    %     weights = w;
    
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end
    
% generalized Jacobi Lump POD method
elseif strcmp(lower(Scheme), 'jl')
    
    Ns = size(Snapshots, 2);
    H = size(Snapshots, 1);
    w = ones(Ns, 1) / Ns;
    weights = w;
    Phi = zeros(size(Snapshots,1), 1);
    b = zeros(Ns, 1);
    JacobiItts = 1; % number of itterations of the jacobi method
    
    %temp_scale=size(Noise,1)^2;
    %TEMP=Noise'*Noise/temp_scale;
    for k=1:Nite
        Lump = Snapshots * w;
        for i = 1:Ns
            b(i) = 1 / H * sum( Snapshots(:, i) .* Lump );
        end
        %ww=inv(TEMP)*b;
        %u=1./diag(TEMP).*b;
        %for itt=1:10
        %    c = TEMP * u;
        %    for i = 1:Ns
        %        u(i) = 1 / TEMP(i,i) * ( b(i) - c(i) ) + u(i);
        %    end
        %end
        %w=u;
        %    [itt norm(w-ww)]
        % for k = 1:JacobiItts
        %    c = Noise * w;
        %    for i = 1:Ns
        %        w(i) = 1 / sum( Noise(:, i) .^2 ) * ( b(i) - sum( Noise(:, i) .* c) ) + w(i);
        %    end
        %end
        w = JacobiOne(Noise, w, b);
        w = w / sum(abs(w));w = w / sum(w);
        weights = [weights w];
    end
    
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end 
    
% generalized Guass-Seidell Lump POD method
elseif strcmp(lower(Scheme), 'gl')   
    
    Ns = size(Snapshots, 2);
    w = ones(Ns, 1) / Ns;
    weights = w;
    Phi = zeros(size(Snapshots,1), 1);
    b = zeros(Ns, 1);
    H = size(Snapshots, 1);
    
    for k=1:Nite
        Lump = Snapshots * w;
        for i = 1:Ns
            b(i) = 1 / H * sum( Snapshots(:, i) .* Lump );
        end
        %c = Noise * w - Noise(:, 1) * w(1);
        %w(1) = 1 / sum( Noise(:, 1) .^2 ) * ( b(1) - sum( Noise(:, 1) .* c) );
        %for i = 2:Ns
        %    c = c + w(i-1) * Noise(:, i-1) - w(i) * Noise(:, i);
        %    w(i) = 1 / sum( Noise(:, i) .^2 ) * ( b(i) - sum( Noise(:, i) .* c) );
        %end
        w = GaussOne(Noise, w, b);
        w = w / sum(abs(w));w = w / sum(w);
        weights = [weights w];
    end

    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end   

% Generalized Adaptive Jacobi POD method
elseif strcmp(lower(Scheme), 'gja')
    
    Ns = size(Snapshots, 2);
    height = size(Snapshots, 1);
    subheight = floor(height / 52);
    w = ones(Ns, 1) / Ns;
    b = zeros(Ns, 1);
    b_old = b;
    weights = w / sum(abs(w));
    Phi = zeros(size(Snapshots, 1), 1);
    
    % perform all the full sized itterations of the adaptive method
    for k = 1:floor(height / subheight)
        % equivilant - matrix multiplication is same as adding weighted columns 
        Lump = Snapshots((k-1)*subheight+1:k*subheight, :) * w;
        c = Noise((k-1)*subheight+1:k*subheight, :) * w;
		  %d = d/norm(d); c = c/norm(c);
        % add new sum to accumulated eigenvector with appropriate weights
        for i = 1:Ns
           	s = 1 / subheight * sum(Snapshots((k-1)*subheight+1:k*subheight, i) .* Lump);
            b(i) = (k-1)/k*w(i) + 1/k*s;
            %for itt = 1:5
            %	w(i) = 1 / sum( Noise((k-1)*subheight+1:k*subheight, i).^2 ) * ( subheight * b(i) - sum( Noise((k-1)*subheight+1:k*subheight, i).* c) ) + w(i);
            %end
        end 
        w = JacobiOne(Noise((k-1)*subheight+1:k*subheight, :), w, k*b - (k-1)*b_old);
        b_old = b;
        % save the weight at this itteration
        weig = w / sum(abs(w)); weig = weig / sum(weig);
        weights = [weights weig];
    end
    
    % perform the last itteration in case subheight does not divide height
    if height - k*subheight ~= height - k*subheight  %height - k*subheight ~= height - k*subheight
       	d = zeros(height - k*subheight, 1);
       	c = zeros(height - k*subheight, 1);
        	for i = 1:Ns
           d = d + w(i) * Snapshots(k*subheight+1:k*subheight+length(d), i);
           c = c + w(i) * Noise(k*subheight+1:k*subheight+length(c), i);
        	end
        	% add new sum to accumulated eigenvector with appropriate weights
        	% adjust weight to accomidate the smaller size of the last peice
        	for i = 1:Ns
           	s = sum(Snapshots(k*subheight+1:height, i) .* d);
            b(i) = b(i) + (s / norm(s)) * (height - i*subheight)/subheight;  
            %w(i) = b(i); %1 / sum( Noise(k*subheight+1:height, i).^2 ) * ( b(i) - sum( Noise(k*subheight+1:height, i).* c) ) + w(i);
         end
         
        weights = [weights w/sum(abs(w))];
    end
    
    % normalize weights
    w = w / sum(abs(w));w = w / sum(w);
    
    for i = 1:Ns
        Phi = Phi + w(i) * Snapshots(:, i);
    end
    
% Generalized Adaptive Gauss POD method
elseif strcmp(lower(Scheme), 'gga')
    
    Ns = size(Snapshots, 2);
    height = size(Snapshots, 1);
    subheight = floor(height / Nite);
    w = ones(Ns, 1) / Ns;
    b = zeros(Ns, 1);
    b_old = b;
    weights = w / sum(abs(w));
    Phi = zeros(size(Snapshots, 1), 1);
    nb = 0; nb1=0; nb2=0; s1=0; w1 = 0;

    % perform all the full sized itterations of the adaptive method
    for i = 1:floor(height / subheight)
        % do one itteration of power method with the new theta
        % find the sum(w(n)V(n)) 
        % equivilant - matrix multiplication is same as adding weighted columns 
        Lump =  Snapshots((i-1)*subheight+1:i*subheight, :) * w;
        % add new sum to accumulated eigenvector with appropriate weights
        for k = 1:Ns
            s(k) = 1 / subheight * sum(Snapshots((i-1)*subheight+1:i*subheight, k) .* Lump);
            %b(k) = (i-1)/(i) * w(k) + 1/(i) * s;   
        end
        %s = s / sum(abs(s));
        for k = 1:Ns
            b(k) = (i-1)/(i) * b(k) + 1/(i) * s(k);
        end
        w = GaussOne(Noise((i-1)*subheight+1:i*subheight, :), w, (i)*b - (i-1)*b_old);              
        %nb = [nb norm((i+1)*b - i*b_old)];
        %nb1 = [nb1 norm(b)];
        %nb2 = [nb2 norm(b_old)];
        %s1 = [s1 norm(s)];
        b_old = b;
        %w = w / sum(abs(w));w = w / sum(w);
        w1 =[w1 norm(w)];
        % save the weight at this itteration
        weig = w/sum(abs(w)); weig = weig / sum(weig);
        weights = [weights weig];
    end
    w = w / sum(abs(w)); w = w / sum(w);

	%[[1:Nite+1]' nb1' 100*s1' w1']
    
    % perform the last itteration in case subheight does not divide height
    if height - i*subheight ~= height - i*subheight %height - i*subheight ~= 0
        c = zeros(height - i*subheight, 1);
        c = Snapshots(i*subheight+1:i*subheight+length(c), :) * w;
        c = c * (height - i*subheight)/subheight;  

        % add new sum to accumulated eigenvector with appropriate weights
        % adjust weight to accomidate the smaller size of the last peice
        for k = 1:Ns
            s = 1 / height * sum(Snapshots(i*subheight+1:height, k) .* c);
				b(k) = (i+1)/(i+2)*w(k) + 1/(i+2)*s;   
        end
        w = GaussOne(Noise(i*subheight+1:height, :), w, (i+2)*b*(height - i*subheight)/subheight - (i+1)*b_old);
        b_old = b;
        w = w / sum(abs(w));w = w / sum(w);
        weights = [weights w/sum(abs(w))/sum(w)];
    end
    
    % normalize weights
    w = w / sum(abs(w));w = w / sum(w);
    for k = 1:Ns
        Phi = Phi + w(k) * Snapshots(:, k);
    end
    
    
% Incorrect scheme type
else
    fprintf('\nError in file POD: Valid Scheme Type Needed\n\m')
    Phi = zeros(size(Snapshots, 1), 1);
    weights = 0;
end